@@ -86,7 +86,7 @@ typedef struct column_t
 } column;
 
 /* counts the number of occurrences of the character in the string */
-int strnchr(const char *s, char c)
+static int strnchr(const char *s, char c)
 {
     int n = 0;
 
@@ -103,7 +103,7 @@ int strnchr(const char *s, char c)
 
 /* Returns a field given by column n (0-based) in a character-
    delimited string s */
-char *delimited_column(char *s, char delim, int n)
+static char *delimited_column(const char *s, char delim, int n)
 {
     if (strnchr(s, delim) < n)
     {
@@ -137,7 +137,7 @@ char *delimited_column(char *s, char delim, int n)
 }
 
 /* returns the number of decimals in a real number given as a string s */
-int str_to_ndecimals(const char *s)
+static int str_to_ndecimals(const char *s)
 {
     if (s == NULL)
     {
@@ -175,7 +175,7 @@ int str_to_ndecimals(const char *s)
 
 /* Determines the most specific column type.
    The most specific types from most to least are integer, float, string.  */
-DBFFieldType str_to_fieldtype(const char *s)
+static DBFFieldType str_to_fieldtype(const char *s)
 {
     size_t len = strlen(s);
 
@@ -198,7 +198,7 @@ DBFFieldType str_to_fieldtype(const char *s)
 }
 
 /* returns the field width */
-int str_to_nwidth(const char *s, DBFFieldType eType)
+static int str_to_nwidth(const char *s, DBFFieldType eType)
 {
     switch (eType)
     {
@@ -214,7 +214,7 @@ int str_to_nwidth(const char *s, DBFFieldType eType)
 }
 
 /* returns true if f1 is more general than f2, otherwise false */
-int more_general_field_type(DBFFieldType t1, DBFFieldType t2)
+static int more_general_field_type(DBFFieldType t1, DBFFieldType t2)
 {
     if (FTInteger == t2 && t1 != FTInteger)
     {
@@ -229,7 +229,7 @@ int more_general_field_type(DBFFieldType t1, DBFFieldType t2)
     return 0;
 }
 
-void strip_crlf(char *line)
+static void strip_crlf(const char *line)
 {
     /* remove trailing CR/LF */
 
@@ -246,7 +246,7 @@ void strip_crlf(char *line)
     }
 }
 
-static void IGNORE_FGETS_RET_VAL(char* s)
+static void IGNORE_FGETS_RET_VAL(char *s)
 {
     (void)s;
 }
@@ -303,6 +303,7 @@ int main(int argc, char **argv)
     if (delimiter == sbuffer[strlen(sbuffer) - 1])
     {
         fprintf(stderr, "lines must not end with the delimiter character\n");
+        fclose(csv_f);
         return EXIT_FAILURE;
     }
 
@@ -313,6 +314,7 @@ int main(int argc, char **argv)
     if (n_columns > MAX_COLUMNS)
     {
         fprintf(stderr, "too many columns, maximum is %i\n", MAX_COLUMNS);
+        fclose(csv_f);
         return EXIT_FAILURE;
     }
 
@@ -327,6 +329,7 @@ int main(int argc, char **argv)
                     "Number of columns on row %i does not match number of "
                     "columns on row 1\n",
                     n_columns);
+            fclose(csv_f);
             return EXIT_FAILURE;
         }
     }
@@ -361,6 +364,7 @@ int main(int argc, char **argv)
     {
         fprintf(stderr, "The header row must define one each a column named "
                         "longitude and latitude\n");
+        fclose(csv_f);
         return EXIT_FAILURE;
     }
 
@@ -431,12 +435,13 @@ int main(int argc, char **argv)
 
     printf("Initializing output files...\n");
 
-    // TODO(schwehr): Close csv_f, shp_h, and dbf_h before EXIT_FAILURE
     SHPHandle shp_h = SHPCreate(argv[3], SHPT_POINT);
     DBFHandle dbf_h = DBFCreate(argv[3]);
     if (NULL == dbf_h)
     {
         fprintf(stderr, "DBFCreate failed\n");
+        SHPClose(shp_h);
+        fclose(csv_f);
         exit(EXIT_FAILURE);
     }
 
@@ -457,6 +462,9 @@ int main(int argc, char **argv)
                               columns[x].nDecimals))
         {
             fprintf(stderr, "DBFFieldAdd failed column %i\n", x + 1);
+            SHPClose(shp_h);
+            DBFClose(dbf_h);
+            fclose(csv_f);
             exit(EXIT_FAILURE);
         }
     }
@@ -518,6 +526,9 @@ int main(int argc, char **argv)
             if (!b)
             {
                 fprintf(stderr, "DBFWrite*Attribute failed\n");
+                SHPClose(shp_h);
+                DBFClose(dbf_h);
+                fclose(csv_f);
                 exit(EXIT_FAILURE);
             }
         }
