@@ -13,6 +13,8 @@
 
 #include "shapefil_private.h"
 
+#include <assert.h>
+#include <errno.h>
 #include <math.h>
 #include <stdbool.h>
 #include <stdio.h>
@@ -24,6 +26,7 @@
 #include "cpl_string.h"
 #else
 
+#ifndef STRCASECMP
 #if defined(_MSC_VER)
 #define STRCASECMP(a, b) (_stricmp(a, b))
 #elif defined(_WIN32)
@@ -32,6 +35,7 @@
 #include <strings.h>
 #define STRCASECMP(a, b) (strcasecmp(a, b))
 #endif
+#endif
 
 #if defined(_MSC_VER)
 #if _MSC_VER < 1900
@@ -353,10 +357,19 @@ DBFHandle SHPAPI_CALL DBFOpenLL(const char *pszFilename, const char *pszAccess,
     /* -------------------------------------------------------------------- */
     const int nLenWithoutExtension = DBFGetLenWithoutExtension(pszFilename);
     char *pszFullname = STATIC_CAST(char *, malloc(nLenWithoutExtension + 5));
+    if (!pszFullname)
+    {
+        return SHPLIB_NULLPTR;
+    }
     memcpy(pszFullname, pszFilename, nLenWithoutExtension);
     memcpy(pszFullname + nLenWithoutExtension, ".dbf", 5);
 
     DBFHandle psDBF = STATIC_CAST(DBFHandle, calloc(1, sizeof(DBFInfo)));
+    if (!psDBF)
+    {
+        free(pszFullname);
+        return SHPLIB_NULLPTR;
+    }
     psDBF->fp = psHooks->FOpen(pszFullname, pszAccess, psHooks->pvUserData);
     memcpy(&(psDBF->sHooks), psHooks, sizeof(SAHooks));
 
@@ -394,6 +407,14 @@ DBFHandle SHPAPI_CALL DBFOpenLL(const char *pszFilename, const char *pszAccess,
     /* -------------------------------------------------------------------- */
     const int nBufSize = 500;
     unsigned char *pabyBuf = STATIC_CAST(unsigned char *, malloc(nBufSize));
+    if (!pabyBuf)
+    {
+        psDBF->sHooks.FClose(psDBF->fp);
+        if (pfCPG)
+            psHooks->FClose(pfCPG);
+        free(psDBF);
+        return SHPLIB_NULLPTR;
+    }
     if (psDBF->sHooks.FRead(pabyBuf, XBASE_FILEHDR_SZ, 1, psDBF->fp) != 1)
     {
         psDBF->sHooks.FClose(psDBF->fp);
@@ -427,8 +448,17 @@ DBFHandle SHPAPI_CALL DBFOpenLL(const char *pszFilename, const char *pszAccess,
     const int nFields = (nHeadLen - XBASE_FILEHDR_SZ) / XBASE_FLDHDR_SZ;
     psDBF->nFields = nFields;
 
-    /* coverity[tainted_data] */
+    assert(psDBF->nRecordLength < 65536);
     psDBF->pszCurrentRecord = STATIC_CAST(char *, malloc(psDBF->nRecordLength));
+    if (!psDBF->pszCurrentRecord)
+    {
+        psDBF->sHooks.FClose(psDBF->fp);
+        if (pfCPG)
+            psDBF->sHooks.FClose(pfCPG);
+        free(pabyBuf);
+        free(psDBF);
+        return SHPLIB_NULLPTR;
+    }
 
     /* -------------------------------------------------------------------- */
     /*  Figure out the code page from the LDID and CPG                      */
@@ -443,7 +473,8 @@ DBFHandle SHPAPI_CALL DBFOpenLL(const char *pszFilename, const char *pszAccess,
         {
             pabyBuf[n] = '\0';
             psDBF->pszCodePage = STATIC_CAST(char *, malloc(n + 1));
-            memcpy(psDBF->pszCodePage, pabyBuf, n + 1);
+            if (psDBF->pszCodePage)
+                memcpy(psDBF->pszCodePage, pabyBuf, n + 1);
         }
         psDBF->sHooks.FClose(pfCPG);
     }
@@ -453,13 +484,28 @@ DBFHandle SHPAPI_CALL DBFOpenLL(const char *pszFilename, const char *pszAccess,
                  psDBF->iLanguageDriver);
         psDBF->pszCodePage = STATIC_CAST(
             char *, malloc(strlen(REINTERPRET_CAST(char *, pabyBuf)) + 1));
-        strcpy(psDBF->pszCodePage, REINTERPRET_CAST(char *, pabyBuf));
+        if (psDBF->pszCodePage)
+            strcpy(psDBF->pszCodePage, REINTERPRET_CAST(char *, pabyBuf));
     }
 
     /* -------------------------------------------------------------------- */
     /*  Read in Field Definitions                                           */
     /* -------------------------------------------------------------------- */
-    pabyBuf = STATIC_CAST(unsigned char *, realloc(pabyBuf, nHeadLen));
+
+    // To please Coverity Scan
+    assert(nHeadLen < 65536);
+    unsigned char *pabyBufNew =
+        STATIC_CAST(unsigned char *, realloc(pabyBuf, nHeadLen));
+    if (!pabyBufNew)
+    {
+        psDBF->sHooks.FClose(psDBF->fp);
+        free(pabyBuf);
+        free(psDBF->pszCurrentRecord);
+        free(psDBF->pszCodePage);
+        free(psDBF);
+        return SHPLIB_NULLPTR;
+    }
+    pabyBuf = pabyBufNew;
     psDBF->pszHeader = REINTERPRET_CAST(char *, pabyBuf);
 
     psDBF->sHooks.FSeek(psDBF->fp, XBASE_FILEHDR_SZ, 0);
@@ -478,6 +524,12 @@ DBFHandle SHPAPI_CALL DBFOpenLL(const char *pszFilename, const char *pszAccess,
     psDBF->panFieldSize = STATIC_CAST(int *, malloc(sizeof(int) * nFields));
     psDBF->panFieldDecimals = STATIC_CAST(int *, malloc(sizeof(int) * nFields));
     psDBF->pachFieldType = STATIC_CAST(char *, malloc(sizeof(char) * nFields));
+    if (!psDBF->panFieldOffset || !psDBF->panFieldSize ||
+        !psDBF->panFieldDecimals || !psDBF->pachFieldType)
+    {
+        DBFClose(psDBF);
+        return SHPLIB_NULLPTR;
+    }
 
     for (int iField = 0; iField < nFields; iField++)
     {
@@ -625,6 +677,8 @@ DBFHandle SHPAPI_CALL DBFCreateLL(const char *pszFilename,
     /* -------------------------------------------------------------------- */
     const int nLenWithoutExtension = DBFGetLenWithoutExtension(pszFilename);
     char *pszFullname = STATIC_CAST(char *, malloc(nLenWithoutExtension + 5));
+    if (!pszFullname)
+        return SHPLIB_NULLPTR;
     memcpy(pszFullname, pszFilename, nLenWithoutExtension);
     memcpy(pszFullname + nLenWithoutExtension, ".dbf", 5);
 
@@ -634,6 +688,16 @@ DBFHandle SHPAPI_CALL DBFCreateLL(const char *pszFilename,
     SAFile fp = psHooks->FOpen(pszFullname, "wb+", psHooks->pvUserData);
     if (fp == SHPLIB_NULLPTR)
     {
+        const size_t nMessageLen = strlen(pszFullname) + 256;
+        char *pszMessage = STATIC_CAST(char *, malloc(nMessageLen));
+        if (pszMessage)
+        {
+            snprintf(pszMessage, nMessageLen, "Failed to create file %s: %s",
+                     pszFullname, strerror(errno));
+            psHooks->Error(pszMessage);
+            free(pszMessage);
+        }
+
         free(pszFullname);
         return SHPLIB_NULLPTR;
     }
@@ -670,6 +734,10 @@ DBFHandle SHPAPI_CALL DBFCreateLL(const char *pszFilename,
     /*      Create the info structure.                                      */
     /* -------------------------------------------------------------------- */
     DBFHandle psDBF = STATIC_CAST(DBFHandle, calloc(1, sizeof(DBFInfo)));
+    if (!psDBF)
+    {
+        return SHPLIB_NULLPTR;
+    }
 
     memcpy(&(psDBF->sHooks), psHooks, sizeof(SAHooks));
     psDBF->fp = fp;
@@ -697,7 +765,8 @@ DBFHandle SHPAPI_CALL DBFCreateLL(const char *pszFilename,
     {
         psDBF->pszCodePage =
             STATIC_CAST(char *, malloc(strlen(pszCodePage) + 1));
-        strcpy(psDBF->pszCodePage, pszCodePage);
+        if (psDBF->pszCodePage)
+            strcpy(psDBF->pszCodePage, pszCodePage);
     }
     DBFSetLastModifiedDate(psDBF, 95, 7, 26); /* dummy date */
 
@@ -804,20 +873,68 @@ int SHPAPI_CALL DBFAddNativeFieldType(DBFHandle psDBF, const char *pszFieldName,
     /*      realloc all the arrays larger to hold the additional field      */
     /*      information.                                                    */
     /* -------------------------------------------------------------------- */
+
+    int *panFieldOffsetNew =
+        STATIC_CAST(int *, realloc(psDBF->panFieldOffset,
+                                   sizeof(int) * (psDBF->nFields + 1)));
+
+    int *panFieldSizeNew =
+        STATIC_CAST(int *, realloc(psDBF->panFieldSize,
+                                   sizeof(int) * (psDBF->nFields + 1)));
+
+    int *panFieldDecimalsNew =
+        STATIC_CAST(int *, realloc(psDBF->panFieldDecimals,
+                                   sizeof(int) * (psDBF->nFields + 1)));
+
+    char *pachFieldTypeNew =
+        STATIC_CAST(char *, realloc(psDBF->pachFieldType,
+                                    sizeof(char) * (psDBF->nFields + 1)));
+
+    char *pszHeaderNew =
+        STATIC_CAST(char *, realloc(psDBF->pszHeader,
+                                    (psDBF->nFields + 1) * XBASE_FLDHDR_SZ));
+
+    /* -------------------------------------------------------------------- */
+    /*      Make the current record buffer appropriately larger.            */
+    /* -------------------------------------------------------------------- */
+    char *pszCurrentRecordNew =
+        STATIC_CAST(char *, realloc(psDBF->pszCurrentRecord,
+                                    psDBF->nRecordLength + nWidth));
+
+    if (panFieldOffsetNew)
+        psDBF->panFieldOffset = panFieldOffsetNew;
+    if (panFieldSizeNew)
+        psDBF->panFieldSize = panFieldSizeNew;
+    if (panFieldDecimalsNew)
+        psDBF->panFieldDecimals = panFieldDecimalsNew;
+    if (pachFieldTypeNew)
+        psDBF->pachFieldType = pachFieldTypeNew;
+    if (pszHeaderNew)
+        psDBF->pszHeader = pszHeaderNew;
+    if (pszCurrentRecordNew)
+        psDBF->pszCurrentRecord = pszCurrentRecordNew;
+
+    if (!panFieldOffsetNew || !panFieldSizeNew || !panFieldDecimalsNew ||
+        !pachFieldTypeNew || !pszHeaderNew || !pszCurrentRecordNew)
+    {
+        psDBF->sHooks.Error("Out of memory");
+        return -1;
+    }
+
+    /* alloc record */
+    char *pszRecord = SHPLIB_NULLPTR;
+    if (!psDBF->bNoHeader)
+    {
+        pszRecord = STATIC_CAST(char *, malloc(psDBF->nRecordLength + nWidth));
+        if (!pszRecord)
+        {
+            psDBF->sHooks.Error("Out of memory");
+            return -1;
+        }
+    }
+
     psDBF->nFields++;
 
-    psDBF->panFieldOffset = STATIC_CAST(
-        int *, realloc(psDBF->panFieldOffset, sizeof(int) * psDBF->nFields));
-
-    psDBF->panFieldSize = STATIC_CAST(
-        int *, realloc(psDBF->panFieldSize, sizeof(int) * psDBF->nFields));
-
-    psDBF->panFieldDecimals = STATIC_CAST(
-        int *, realloc(psDBF->panFieldDecimals, sizeof(int) * psDBF->nFields));
-
-    psDBF->pachFieldType = STATIC_CAST(
-        char *, realloc(psDBF->pachFieldType, sizeof(char) * psDBF->nFields));
-
     /* -------------------------------------------------------------------- */
     /*      Assign the new field information fields.                        */
     /* -------------------------------------------------------------------- */
@@ -833,9 +950,6 @@ int SHPAPI_CALL DBFAddNativeFieldType(DBFHandle psDBF, const char *pszFieldName,
     psDBF->nHeaderLength += XBASE_FLDHDR_SZ;
     psDBF->bUpdated = FALSE;
 
-    psDBF->pszHeader = STATIC_CAST(
-        char *, realloc(psDBF->pszHeader, psDBF->nFields * XBASE_FLDHDR_SZ));
-
     char *pszFInfo = psDBF->pszHeader + XBASE_FLDHDR_SZ * (psDBF->nFields - 1);
 
     for (int i = 0; i < XBASE_FLDHDR_SZ; i++)
@@ -856,12 +970,6 @@ int SHPAPI_CALL DBFAddNativeFieldType(DBFHandle psDBF, const char *pszFieldName,
         pszFInfo[17] = STATIC_CAST(unsigned char, nDecimals);
     }
 
-    /* -------------------------------------------------------------------- */
-    /*      Make the current record buffer appropriately larger.            */
-    /* -------------------------------------------------------------------- */
-    psDBF->pszCurrentRecord = STATIC_CAST(
-        char *, realloc(psDBF->pszCurrentRecord, psDBF->nRecordLength));
-
     /* we're done if dealing with new .dbf */
     if (psDBF->bNoHeader)
         return (psDBF->nFields - 1);
@@ -870,10 +978,6 @@ int SHPAPI_CALL DBFAddNativeFieldType(DBFHandle psDBF, const char *pszFieldName,
     /*      For existing .dbf file, shift records                           */
     /* -------------------------------------------------------------------- */
 
-    /* alloc record */
-    char *pszRecord =
-        STATIC_CAST(char *, malloc(sizeof(char) * psDBF->nRecordLength));
-
     const char chFieldFill = DBFGetNullCharacter(chType);
 
     SAOffset nRecordOffset;
@@ -1346,14 +1450,14 @@ static bool DBFWriteAttribute(DBFHandle psDBF, int hEntity, int iField,
             char szFormat[20];
             snprintf(szFormat, sizeof(szFormat), "%%%d.%df", nWidth,
                      psDBF->panFieldDecimals[iField]);
-            CPLsnprintf(szSField, sizeof(szSField), szFormat,
-                        *STATIC_CAST(double *, pValue));
+            const double value = *STATIC_CAST(double *, pValue);
+            CPLsnprintf(szSField, sizeof(szSField), szFormat, value);
             szSField[sizeof(szSField) - 1] = '\0';
             if (STATIC_CAST(int, strlen(szSField)) >
                 psDBF->panFieldSize[iField])
             {
                 szSField[psDBF->panFieldSize[iField]] = '\0';
-                nRetResult = false;
+                nRetResult = psDBF->sHooks.Atof(szSField) == value;
             }
             memcpy(REINTERPRET_CAST(char *,
                                     pabyRec + psDBF->panFieldOffset[iField]),
@@ -1781,11 +1885,7 @@ int SHPAPI_CALL DBFMarkRecordDeleted(DBFHandle psDBF, int iShape,
     /* -------------------------------------------------------------------- */
     /*      Assign value, marking record as dirty if it changes.            */
     /* -------------------------------------------------------------------- */
-    char chNewFlag;
-    if (bIsDeleted)
-        chNewFlag = '*';
-    else
-        chNewFlag = ' ';
+    const char chNewFlag = bIsDeleted ? '*' : ' ';
 
     if (psDBF->pszCurrentRecord[0] != chNewFlag)
     {
@@ -1963,6 +2063,31 @@ int SHPAPI_CALL DBFReorderFields(DBFHandle psDBF, const int *panMap)
         STATIC_CAST(char *, calloc(psDBF->nFields, sizeof(char)));
     char *pszHeaderNew = STATIC_CAST(
         char *, malloc(sizeof(char) * XBASE_FLDHDR_SZ * psDBF->nFields));
+    char *pszRecord = SHPLIB_NULLPTR;
+    char *pszRecordNew = SHPLIB_NULLPTR;
+    if (!(psDBF->bNoHeader && psDBF->nRecords == 0))
+    {
+        /* alloc record */
+        pszRecord =
+            STATIC_CAST(char *, malloc(sizeof(char) * psDBF->nRecordLength));
+        pszRecordNew =
+            STATIC_CAST(char *, malloc(sizeof(char) * psDBF->nRecordLength));
+    }
+    if (!panFieldOffsetNew || !panFieldSizeNew || !panFieldDecimalsNew ||
+        !pachFieldTypeNew || !pszHeaderNew ||
+        (!(psDBF->bNoHeader && psDBF->nRecords == 0) &&
+         (!pszRecord || !pszRecordNew)))
+    {
+        free(panFieldOffsetNew);
+        free(panFieldSizeNew);
+        free(panFieldDecimalsNew);
+        free(pachFieldTypeNew);
+        free(pszHeaderNew);
+        free(pszRecord);
+        free(pszRecordNew);
+        psDBF->sHooks.Error("Out of memory");
+        return FALSE;
+    }
 
     /* shuffle fields definitions */
     for (int i = 0; i < psDBF->nFields; i++)
@@ -1992,12 +2117,6 @@ int SHPAPI_CALL DBFReorderFields(DBFHandle psDBF, const int *panMap)
         psDBF->bNoHeader = TRUE;
         DBFUpdateHeader(psDBF);
 
-        /* alloc record */
-        char *pszRecord =
-            STATIC_CAST(char *, malloc(sizeof(char) * psDBF->nRecordLength));
-        char *pszRecordNew =
-            STATIC_CAST(char *, malloc(sizeof(char) * psDBF->nRecordLength));
-
         /* shuffle fields in records */
         for (int iRecord = 0; iRecord < psDBF->nRecords; iRecord++)
         {
@@ -2028,12 +2147,12 @@ int SHPAPI_CALL DBFReorderFields(DBFHandle psDBF, const int *panMap)
             psDBF->sHooks.FWrite(pszRecordNew, psDBF->nRecordLength, 1,
                                  psDBF->fp);
         }
-
-        /* free record */
-        free(pszRecord);
-        free(pszRecordNew);
     }
 
+    /* free record */
+    free(pszRecord);
+    free(pszRecordNew);
+
     if (errorAbort)
     {
         free(panFieldOffsetNew);
@@ -2096,6 +2215,31 @@ int SHPAPI_CALL DBFAlterFieldDefn(DBFHandle psDBF, int iField,
     if (nWidth > XBASE_FLD_MAX_WIDTH)
         nWidth = XBASE_FLD_MAX_WIDTH;
 
+    char *pszRecord = STATIC_CAST(
+        char *, malloc(nOldRecordLength +
+                       ((nWidth > nOldWidth) ? nWidth - nOldWidth : 0)));
+    char *pszOldField = STATIC_CAST(char *, malloc(nOldWidth + 1));
+    if (!pszRecord || !pszOldField)
+    {
+        free(pszRecord);
+        free(pszOldField);
+        return FALSE;
+    }
+
+    if (nWidth != nOldWidth)
+    {
+        char *pszCurrentRecordNew = STATIC_CAST(
+            char *, realloc(psDBF->pszCurrentRecord,
+                            psDBF->nRecordLength + nWidth - nOldWidth));
+        if (!pszCurrentRecordNew)
+        {
+            free(pszRecord);
+            free(pszOldField);
+            return FALSE;
+        }
+        psDBF->pszCurrentRecord = pszCurrentRecordNew;
+    }
+
     /* -------------------------------------------------------------------- */
     /*      Assign the new field information fields.                        */
     /* -------------------------------------------------------------------- */
@@ -2134,14 +2278,15 @@ int SHPAPI_CALL DBFAlterFieldDefn(DBFHandle psDBF, int iField,
         for (int i = iField + 1; i < psDBF->nFields; i++)
             psDBF->panFieldOffset[i] += nWidth - nOldWidth;
         psDBF->nRecordLength += nWidth - nOldWidth;
-
-        psDBF->pszCurrentRecord = STATIC_CAST(
-            char *, realloc(psDBF->pszCurrentRecord, psDBF->nRecordLength));
     }
 
     /* we're done if we're dealing with not yet created .dbf */
     if (psDBF->bNoHeader && psDBF->nRecords == 0)
+    {
+        free(pszRecord);
+        free(pszOldField);
         return TRUE;
+    }
 
     /* force update of header with new header and record length */
     psDBF->bNoHeader = TRUE;
@@ -2151,11 +2296,6 @@ int SHPAPI_CALL DBFAlterFieldDefn(DBFHandle psDBF, int iField,
 
     if (nWidth < nOldWidth || (nWidth == nOldWidth && chType != chOldType))
     {
-        char *pszRecord =
-            STATIC_CAST(char *, malloc(sizeof(char) * nOldRecordLength));
-        char *pszOldField =
-            STATIC_CAST(char *, malloc(sizeof(char) * (nOldWidth + 1)));
-
         pszOldField[nOldWidth] = 0;
 
         /* move records to their new positions */
@@ -2223,17 +2363,9 @@ int SHPAPI_CALL DBFAlterFieldDefn(DBFHandle psDBF, int iField,
             psDBF->sHooks.FWrite(&ch, 1, 1, psDBF->fp);
         }
         /* TODO: truncate file */
-
-        free(pszRecord);
-        free(pszOldField);
     }
     else if (nWidth > nOldWidth)
     {
-        char *pszRecord =
-            STATIC_CAST(char *, malloc(sizeof(char) * psDBF->nRecordLength));
-        char *pszOldField =
-            STATIC_CAST(char *, malloc(sizeof(char) * (nOldWidth + 1)));
-
         pszOldField[nOldWidth] = 0;
 
         /* move records to their new positions */
@@ -2296,7 +2428,7 @@ int SHPAPI_CALL DBFAlterFieldDefn(DBFHandle psDBF, int iField,
 
         if (!errorAbort && psDBF->bWriteEndOfFileChar)
         {
-            char ch = END_OF_FILE_CHARACTER;
+            const char ch = END_OF_FILE_CHARACTER;
 
             SAOffset nRecordOffset =
                 psDBF->nRecordLength * STATIC_CAST(SAOffset, psDBF->nRecords) +
@@ -2305,11 +2437,11 @@ int SHPAPI_CALL DBFAlterFieldDefn(DBFHandle psDBF, int iField,
             psDBF->sHooks.FSeek(psDBF->fp, nRecordOffset, 0);
             psDBF->sHooks.FWrite(&ch, 1, 1, psDBF->fp);
         }
-
-        free(pszRecord);
-        free(pszOldField);
     }
 
+    free(pszRecord);
+    free(pszOldField);
+
     if (errorAbort)
     {
         psDBF->nCurrentRecord = -1;
